Package: checkstring
Title: Common String Format Validation
Version: 0.2.0
Authors@R: 
    person("Maximilian", "Mücke", , "muecke.maximilian@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0009-0000-9432-9795"))
Description: Validates common string formats including financial
    identifiers (ISIN, CUSIP, SEDOL, FIGI, IBAN, LEI), publication
    identifiers (ISBN, ISSN, DOI, ORCID), and general formats (email,
    UUID, URL, semver), with check digit verification where applicable.
License: MIT + file LICENSE
URL: https://m-muecke.github.io/checkstring/,
        https://github.com/m-muecke/checkstring
BugReports: https://github.com/m-muecke/checkstring/issues
Depends: R (>= 4.1.0)
Suggests: curl, testthat (>= 3.0.0)
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-06-29 09:16:12 UTC; mmuecke
Author: Maximilian Mücke [aut, cre] (ORCID:
    <https://orcid.org/0009-0000-9432-9795>)
Maintainer: Maximilian Mücke <muecke.maximilian@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-29 09:30:02 UTC

# Generated by roxygen2: do not edit by hand

export(assert_empty_dots)
export(is_base64)
export(is_base64url)
export(is_bic)
export(is_cuid2)
export(is_cusip)
export(is_doi)
export(is_email)
export(is_figi)
export(is_hex)
export(is_hostname)
export(is_iban)
export(is_ipv4)
export(is_isbn)
export(is_isin)
export(is_issn)
export(is_lei)
export(is_mac)
export(is_md5)
export(is_nanoid)
export(is_orcid)
export(is_sedol)
export(is_semver)
export(is_sha256)
export(is_ulid)
export(is_url)
export(is_uuid)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# checkstring

<!-- badges: start -->

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![R-CMD-check](https://github.com/m-muecke/checkstring/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/m-muecke/checkstring/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/checkstring)](https://CRAN.R-project.org/package=checkstring)
[![R-universe
version](https://m-muecke.r-universe.dev/checkstring/badges/version)](https://m-muecke.r-universe.dev/checkstring)
<!-- badges: end -->

The goal of checkstring is to provide string format validation functions
for R, inspired by [Zod’s](https://zod.dev/?id=strings) string
validators. It extends the
[checkmate](https://mllg.github.io/checkmate/) family of argument checks
with common string format validations such as email addresses, UUIDs,
URLs, IP addresses, and financial and academic identifiers like ISIN,
CUSIP, IBAN, ISBN, and ORCID, including check digit verification.

## Installation

You can install the released version of **checkstring** from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("checkstring")
```

And the development version from [GitHub](https://github.com/) with:

``` r
# install.packages("pak")
pak::pak("m-muecke/checkstring")
```

## Usage

``` r
library(checkstring)

is_email("user@example.com")
#> [1] TRUE
is_uuid("550e8400-e29b-41d4-a716-446655440000")
#> [1] TRUE
is_ipv4("192.168.1.1")
#> [1] TRUE
is_hex("deadbeef")
#> [1] TRUE
is_url("https://example.com")
#> [1] TRUE
is_base64("SGVsbG8gV29ybGQ=")
#> [1] TRUE
is_isin("US0378331005")
#> [1] TRUE
is_iban("GB29NWBK60161331926819")
#> [1] TRUE
```

## Related work

- [checkmate](https://github.com/mllg/checkmate): Fast and versatile
  argument checks for R
- [figir](https://github.com/philaris/figir): Check validity of FIGI,
  CUSIP, ISIN, SEDOL
- [libbib](https://github.com/NYPL/libbib): Validate and normalize ISBN,
  ISSN, LCCN

#' Assert that dots are empty
#'
#' @param ... (`any`)\cr
#'   Ellipsis arguments to check.
#' @return `NULL` invisibly if empty, otherwise throws an error.
#' @examples
#' assert_empty_dots()
#' @export
assert_empty_dots <- function(...) {
  n <- ...length()
  if (n == 0L) {
    return(invisible())
  }
  nms <- ...names()
  if (is.null(nms)) {
    stop(sprintf("Received %i unused unnamed argument(s).", n), call. = FALSE)
  }
  named <- nms[nzchar(nms)]
  n_unnamed <- n - length(named)
  if (n_unnamed == 0L) {
    stop(sprintf("Received unused named argument(s): %s.", toString(named)), call. = FALSE)
  }
  stop(
    sprintf(
      "Received %i unused unnamed argument(s) and unused named argument(s): %s.",
      n_unnamed,
      toString(named)
    ),
    call. = FALSE
  )
}

is_string <- function(x) {
  is.character(x) && length(x) == 1L && !is.na(x)
}

test_that("assert_empty_dots works", {
  expect_invisible(assert_empty_dots())
  expect_null(assert_empty_dots())
  expect_error(
    assert_empty_dots(1L, 2L, 3L),
    "Received 3 unused unnamed argument(s).",
    fixed = TRUE
  )
  expect_error(
    assert_empty_dots(a = 1L, b = 2L),
    "Received unused named argument(s): a, b.",
    fixed = TRUE
  )
  expect_error(
    assert_empty_dots(1L, 2L, a = 3L, b = 4L),
    "Received 2 unused unnamed argument(s) and unused named argument(s): a, b.",
    fixed = TRUE
  )
})

#' Check if an argument is an ISO 8601 date string
#'
#' Validates the `YYYY-MM-DD` format with calendar correctness, including leap year handling.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid ISO 8601 date string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/ISO_8601>
#' @examples
#' is_iso_date("2024-01-15")
#' is_iso_date("2024-02-30")
#' @export
is_iso_date <- function(x) {
  if (!is_string(x) || !grepl("^\\d{4}-\\d{2}-\\d{2}$", x)) {
    return(FALSE)
  }
  parts <- as.integer(strsplit(x, "-", fixed = TRUE)[[1L]])
  year <- parts[[1L]]
  month <- parts[[2L]]
  day <- parts[[3L]]
  if (month < 1L || month > 12L || day < 1L) {
    return(FALSE)
  }
  days_in_month <- c(31L, 28L, 31L, 30L, 31L, 30L, 31L, 31L, 30L, 31L, 30L, 31L)
  if (month == 2L && (year %% 4L == 0L && (year %% 100L != 0L || year %% 400L == 0L))) {
    days_in_month[[2L]] <- 29L
  }
  day <= days_in_month[[month]]
}

#' Check if an argument is an ISO 8601 datetime string
#'
#' Validates the `YYYY-MM-DDTHH:MM:SS[.sss]` format with calendar correctness, including leap year
#' handling. The timezone designator is optional and can be `Z` or an offset like `+05:30`.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid ISO 8601 datetime string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/ISO_8601>
#' @examples
#' is_iso_datetime("2024-01-15T12:00:00Z")
#' is_iso_datetime("2024-01-15T12:00:00+05:30")
#' is_iso_datetime("2024-01-15T12:00:00")
#' @export
is_iso_datetime <- function(x) {
  regex <- "^(\\d{4}-\\d{2}-\\d{2})T(\\d{2}):(\\d{2}):(\\d{2})(\\.\\d+)?(?:Z|[+-]\\d{2}:\\d{2})?$"
  if (!is_string(x) || !grepl(regex, x, perl = TRUE)) {
    return(FALSE)
  }
  m <- regmatches(x, regexec(regex, x, perl = TRUE))[[1L]]
  if (!is_iso_date(m[[2L]])) {
    return(FALSE)
  }
  hour <- as.integer(m[[3L]])
  min <- as.integer(m[[4L]])
  sec <- as.integer(m[[5L]])
  hour <= 23L && min <= 59L && sec <= 59L
}

#' Check if an argument is a hex color string
#'
#' Validates hex color codes in `#RGB`, `#RGBA`, `#RRGGBB`, or `#RRGGBBAA` format.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid hex color string, `FALSE` otherwise.
#' @references
#' <https://www.w3.org/TR/css-color-4/#hex-notation>
#' @examples
#' is_color_hex("#FF5733")
#' is_color_hex("#fff")
#' @export
is_color_hex <- function(x) {
  is_string(x) && grepl("^#([0-9a-f]{3,4}|[0-9a-f]{6}|[0-9a-f]{8})$", x, ignore.case = TRUE)
}

#' Check if an argument is a email address string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid email address string, `FALSE` otherwise.
#' @examples
#' is_email("user@example.com")
#' @export
is_email <- function(x) {
  regex <- "^(?!\\.)(?!.*\\.\\.)([A-Z0-9_'+\\-\\.]*[A-Z0-9_+\\-])@([A-Z0-9][A-Z0-9\\-]*\\.)+[A-Z]{2,}$" # nolint
  is_string(x) && grepl(regex, x, ignore.case = TRUE, perl = TRUE)
}

#' Check if an argument is an UUID string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid UUID string, `FALSE` otherwise.
#' @examples
#' is_uuid("550e8400-e29b-41d4-a716-446655440000")
#' @export
is_uuid <- function(x) {
  regex <- "^[0-9a-f]{8}\\b-[0-9a-f]{4}\\b-[0-9a-f]{4}\\b-[0-9a-f]{4}\\b-[0-9a-f]{12}$" # nolint
  is_string(x) && grepl(regex, x, ignore.case = TRUE, perl = TRUE)
}

#' Check if an argument is a MIME type string
#'
#' Validates the `type/subtype` format where the top-level type must be one of the IANA-registered
#' types: `application`, `audio`, `font`, `image`, `message`, `model`, `multipart`, `text`, or
#' `video`.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid MIME type string, `FALSE` otherwise.
#' @references
#' <https://www.iana.org/assignments/media-types/media-types.xhtml>
#' @examples
#' is_mime("application/json")
#' is_mime("text/plain")
#' @export
is_mime <- function(x) {
  types <- "application|audio|font|image|message|model|multipart|text|video"
  regex <- paste0("^(", types, ")/[a-zA-Z0-9][a-zA-Z0-9!#$&\\-^_.+]*$")
  is_string(x) && grepl(regex, x, ignore.case = TRUE, perl = TRUE)
}

#' Check if an argument is a base64 string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid base64 string, `FALSE` otherwise.
#' @examples
#' is_base64("SGVsbG8gV29ybGQ=")
#' @export
is_base64 <- function(x) {
  regex <- "^([0-9a-zA-Z+/]{4})*((([0-9a-zA-Z+/]{2}==)|([0-9a-zA-Z+/]{3}=))?)$"
  is_string(x) && grepl(regex, x, perl = TRUE)
}

#' Check if an argument is base64url string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid base64url string, `FALSE` otherwise.
#' @examples
#' is_base64url("SGVsbG8gV29ybGQ")
#' @export
is_base64url <- function(x) {
  regex <- "^([0-9a-zA-Z\\-_]{4})*((([0-9a-zA-Z\\-_]{2}(==)?)|([0-9a-zA-Z\\-_]{3}(=)?))?)$" # nolint
  is_string(x) && grepl(regex, x, perl = TRUE)
}

#' Check if an argument is url string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid URL string, `FALSE` otherwise.
#' @examples
#' is_url("https://example.com")
#' @export
is_url <- function(x) {
  if (!requireNamespace("curl", quietly = TRUE)) {
    stop("Please install the curl package to use this function.", call. = FALSE)
  }
  if (!is_string(x) || !nzchar(x)) {
    return(FALSE)
  }
  tryCatch(
    {
      curl::curl_parse_url(x, decode = FALSE)
      TRUE
    },
    error = function(e) FALSE
  )
}

#' Check if an argument is an IPv4 address string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid IPv4 address string, `FALSE` otherwise.
#' @examples
#' is_ipv4("192.168.1.1")
#' @export
is_ipv4 <- function(x) {
  octet <- "(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])"
  regex <- paste0("^", paste(rep(octet, 4L), collapse = "\\."), "$")
  is_string(x) && grepl(regex, x, perl = TRUE)
}

#' Check if an argument is an IPv6 address string
#'
#' Validates full, compressed (`::`) and IPv4-embedded (`::ffff:192.168.1.1`) forms.
#' Zone IDs (e.g. `%eth0`) are not supported.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid IPv6 address string, `FALSE` otherwise.
#' @examples
#' is_ipv6("2001:0db8:85a3:0000:0000:8a2e:0370:7334")
#' is_ipv6("::1")
#' @export
is_ipv6 <- function(x) {
  if (!is_string(x) || !nzchar(x)) {
    return(FALSE)
  }
  if (grepl("[^0-9a-f:.]", x, ignore.case = TRUE)) {
    return(FALSE)
  }
  # at most one "::"; reject ":::" or longer colon runs
  if (grepl(":::", x, fixed = TRUE) || length(gregexpr("::", x, fixed = TRUE)[[1L]]) > 1L) {
    return(FALSE)
  }
  has_compressed <- grepl("::", x, fixed = TRUE)
  # reject lone leading/trailing single colons (not part of "::")
  if (!has_compressed && (startsWith(x, ":") || endsWith(x, ":"))) {
    return(FALSE)
  }
  # split off optional embedded IPv4 tail (e.g. "::ffff:192.168.1.1")
  target_groups <- 8L
  if (grepl("\\.", x)) {
    m <- regmatches(x, regexpr("[^:]+$", x))
    if (!is_ipv4(m)) {
      return(FALSE)
    }
    x <- substr(x, 1L, nchar(x) - nchar(m))
    target_groups <- 6L
    # x now ends in ":"; trim it unless that would destroy a "::"
    if (!endsWith(x, "::")) {
      x <- substr(x, 1L, nchar(x) - 1L)
    }
  }
  hextet <- "^[0-9a-f]{1,4}$"
  if (!has_compressed) {
    groups <- strsplit(x, ":", fixed = TRUE)[[1L]]
    length(groups) == target_groups && all(grepl(hextet, groups, ignore.case = TRUE))
  } else {
    parts <- strsplit(x, "::", fixed = TRUE)[[1L]]
    left <- if (length(parts) >= 1L && nzchar(parts[1L])) {
      strsplit(parts[1L], ":", fixed = TRUE)[[1L]]
    } else {
      character()
    }
    right <- if (length(parts) == 2L && nzchar(parts[2L])) {
      strsplit(parts[2L], ":", fixed = TRUE)[[1L]]
    } else {
      character()
    }
    length(left) + length(right) < target_groups &&
      all(grepl(hextet, left, ignore.case = TRUE)) &&
      all(grepl(hextet, right, ignore.case = TRUE))
  }
}

#' Check if an argument is a hexadecimal string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid non-empty hexadecimal string, `FALSE`
#'   otherwise.
#' @examples
#' is_hex("deadbeef")
#' @export
is_hex <- function(x) {
  is_string(x) && grepl("^[0-9a-f]+$", x, ignore.case = TRUE)
}

#' Check if an argument is an MD5 hash string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid MD5 hash string, `FALSE` otherwise.
#' @examples
#' is_md5("d41d8cd98f00b204e9800998ecf8427e")
#' @export
is_md5 <- function(x) {
  is_string(x) && grepl("^[0-9a-f]{32}$", x, ignore.case = TRUE)
}

#' Check if an argument is a SHA-256 hash string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid SHA-256 hash string, `FALSE` otherwise.
#' @examples
#' is_sha256("e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855")
#' @export
is_sha256 <- function(x) {
  is_string(x) && grepl("^[0-9a-f]{64}$", x, ignore.case = TRUE)
}

#' Check if an argument is a semantic versioning string
#'
#' Validates semantic versioning 2.0.0 format, including optional pre-release and build metadata.
#' Uses the official recommended regex from <https://semver.org/>.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid semver string, `FALSE` otherwise.
#' @references
#' <https://semver.org/>
#' @examples
#' is_semver("1.0.0")
#' @export
is_semver <- function(x) {
  regex <- "^(0|[1-9]\\d*)\\.(0|[1-9]\\d*)\\.(0|[1-9]\\d*)(?:-((?:0|[1-9]\\d*|\\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\\.(?:0|[1-9]\\d*|\\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\\+([0-9a-zA-Z-]+(?:\\.[0-9a-zA-Z-]+)*))?$" # nolint
  is_string(x) && grepl(regex, x, perl = TRUE)
}

#' Check if an argument is a hostname string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid hostname string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/Hostname>
#' @examples
#' is_hostname("example.com")
#' @export
is_hostname <- function(x) {
  regex <- "^([a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?\\.)*[a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?$" # nolint
  is_string(x) && nchar(x) <= 253L && grepl(regex, x, ignore.case = TRUE)
}

#' Check if an argument is a MAC address string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid MAC address string, `FALSE` otherwise.
#' @examples
#' is_mac("00:1B:44:11:3A:B7")
#' @export
is_mac <- function(x) {
  regex <- "^[0-9a-f]{2}([:-])[0-9a-f]{2}(\\1[0-9a-f]{2}){4}$"
  is_string(x) && grepl(regex, x, ignore.case = TRUE)
}

#' Check if an argument is a ULID string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid ULID string, `FALSE` otherwise.
#' @references
#' <https://github.com/ulid/spec>
#' @examples
#' is_ulid("01ARZ3NDEKTSV4RRFFQ69G5FAV")
#' @export
is_ulid <- function(x) {
  is_string(x) && grepl("^[0-9A-HJKMNP-TV-Z]{26}$", x)
}

#' Check if an argument is a Nano ID string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid Nano ID string, `FALSE` otherwise.
#' @examples
#' is_nanoid("V1StGXR8_Z5jdHi6B-myT")
#' @export
is_nanoid <- function(x) {
  is_string(x) && grepl("^[A-Za-z0-9_-]{21}$", x)
}

#' Check if an argument is a CUID2 string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid CUID2 string, `FALSE` otherwise.
#' @examples
#' is_cuid2("ckopqwooh000001la8mbi2im9")
#' @export
is_cuid2 <- function(x) {
  is_string(x) && grepl("^[a-z][0-9a-z]{23,}$", x)
}

test_that("is_iso_date works", {
  expect_true(is_iso_date("2024-01-15"))
  expect_true(is_iso_date("2024-02-29")) # leap year
  expect_true(is_iso_date("2000-02-29")) # divisible by 400
  expect_true(is_iso_date("0001-01-01"))

  expect_false(is_iso_date("2024-02-30")) # feb too many days
  expect_false(is_iso_date("2023-02-29")) # not a leap year
  expect_false(is_iso_date("1900-02-29")) # divisible by 100 but not 400
  expect_false(is_iso_date("2024-13-01")) # invalid month
  expect_false(is_iso_date("2024-00-01")) # month zero
  expect_false(is_iso_date("2024-01-00")) # day zero
  expect_false(is_iso_date("2024-1-15")) # unpadded month
  expect_false(is_iso_date("24-01-15")) # short year
  expect_false(is_iso_date("2024/01/15")) # wrong separator
  expect_false(is_iso_date(1L))
})

test_that("is_iso_datetime works", {
  expect_true(is_iso_datetime("2024-01-15T12:00:00Z"))
  expect_true(is_iso_datetime("2024-01-15T00:00:00Z"))
  expect_true(is_iso_datetime("2024-01-15T23:59:59Z"))
  expect_true(is_iso_datetime("2024-01-15T12:00:00.123Z"))
  expect_true(is_iso_datetime("2024-01-15T12:00:00.123456Z"))
  expect_true(is_iso_datetime("2024-01-15T12:00:00")) # local time
  expect_true(is_iso_datetime("2024-01-15T12:00:00+05:30")) # offset
  expect_true(is_iso_datetime("2024-01-15T12:00:00-08:00"))
  expect_true(is_iso_datetime("2024-01-15T12:00:00.123+05:30"))

  expect_false(is_iso_datetime("2024-01-15T24:00:00Z")) # hour 24
  expect_false(is_iso_datetime("2024-01-15T12:60:00Z")) # minute 60
  expect_false(is_iso_datetime("2024-01-15T12:00:60Z")) # second 60
  expect_false(is_iso_datetime("2024-02-30T12:00:00Z")) # invalid date
  expect_false(is_iso_datetime("2024-01-15 12:00:00Z")) # space instead of T
  expect_false(is_iso_datetime("2024-01-15T12:00:00+5:30")) # unpadded offset hour
  expect_false(is_iso_datetime("2024-01-15T12:00:00+05")) # missing offset minutes
  expect_false(is_iso_datetime(1L))
})

test_that("is_color_hex works", {
  expect_true(is_color_hex("#fff"))
  expect_true(is_color_hex("#FFF"))
  expect_true(is_color_hex("#fffa"))
  expect_true(is_color_hex("#FF5733"))
  expect_true(is_color_hex("#ff5733aa"))
  expect_true(is_color_hex("#000000"))

  expect_false(is_color_hex("FF5733")) # missing #
  expect_false(is_color_hex("#ff")) # too short
  expect_false(is_color_hex("#fffff")) # 5 chars
  expect_false(is_color_hex("#fffffff")) # 7 chars
  expect_false(is_color_hex("#gggggg")) # invalid hex
  expect_false(is_color_hex(1L))
})

test_that("is_email works", {
  expect_true(is_email("user.name+tag@example.com"))
  expect_true(is_email("USER_123@sub.domain.co"))
  expect_true(is_email("a_b-c.d+e@domain-name.com"))

  expect_false(is_email(".username@example.com")) # starts with dot
  expect_false(is_email("user..name@example.com")) # consecutive dots
  expect_false(is_email("user.@example.com")) # ends local part with dot
  expect_false(is_email("user@.com")) # domain starts with dot
  expect_false(is_email("user@com")) # no dot in domain
  expect_false(is_email("user@domain..com")) # consecutive dots in domain
  expect_false(is_email("user@domain.c")) # TLD too short
})

test_that("is_mime works", {
  expect_true(is_mime("application/json"))
  expect_true(is_mime("text/plain"))
  expect_true(is_mime("image/png"))
  expect_true(is_mime("application/vnd.api+json"))
  expect_true(is_mime("audio/mpeg"))
  expect_true(is_mime("multipart/form-data"))
  expect_true(is_mime("font/woff2"))
  expect_true(is_mime("model/gltf+json"))

  expect_false(is_mime("text")) # missing subtype
  expect_false(is_mime("text/")) # empty subtype
  expect_false(is_mime("foo/plain")) # invalid top-level type
  expect_false(is_mime("text/plain; charset=utf-8")) # parameters not allowed
  expect_true(is_mime("TEXT/plain")) # case-insensitive per RFC 2045
  expect_false(is_mime(1L))
})

test_that("is_uuid works", {
  expect_true(is_uuid("123e4567-e89b-12d3-a456-426614174000"))
  expect_true(is_uuid("550e8400-e29b-41d4-a716-446655440000"))

  expect_false(is_uuid("123e4567e89b12d3a456426614174000")) # missing hyphens
  expect_false(is_uuid("123e4567-e89b-12d3-a456-42661417400")) # too short
  expect_false(is_uuid("123e4567-e89b-12d3-a456-4266141740000")) # too long
  expect_false(is_uuid("g23e4567-e89b-12d3-a456-426614174000")) # invalid hex char
})

test_that("is_base64 works", {
  expect_true(is_base64("TWFu"))
  expect_true(is_base64("TWE="))
  expect_true(is_base64("TQ=="))
  expect_true(is_base64("U29tZSBkYXRhIHdpdGggL3VzZXJzLw=="))

  expect_false(is_base64("TWE")) # not padded
  expect_false(is_base64("TWE===")) # overpadded
  expect_false(is_base64("TWE*")) # invalid char
})

test_that("is_base64url works", {
  expect_true(is_base64url("TWFu"))
  expect_true(is_base64url("TWE"))
  expect_true(is_base64url("TQ"))
  expect_true(is_base64url("U29tZV9kYXRhLXdpdGgtL3VzZXJzLw"))

  expect_false(is_base64url("TWE===")) # overpadded
  expect_false(is_base64url("TWE*")) # invalid char
})

test_that("is_ipv4 works", {
  expect_true(is_ipv4("0.0.0.0"))
  expect_true(is_ipv4("127.0.0.1"))
  expect_true(is_ipv4("192.168.1.1"))
  expect_true(is_ipv4("255.255.255.255"))

  expect_false(is_ipv4("256.0.0.1")) # octet > 255
  expect_false(is_ipv4("192.168.1")) # only 3 octets
  expect_false(is_ipv4("192.168.1.1.1")) # 5 octets
  expect_false(is_ipv4("192.168.01.1")) # leading zero
  expect_false(is_ipv4("abc.def.ghi.jkl")) # non-numeric
  expect_false(is_ipv4(1L))
})

test_that("is_ipv6 works", {
  expect_true(is_ipv6("2001:0db8:85a3:0000:0000:8a2e:0370:7334"))
  expect_true(is_ipv6("fe80:0000:0000:0000:0000:0000:0000:0001"))
  expect_true(is_ipv6("FE80::1"))
  expect_true(is_ipv6("::1"))
  expect_true(is_ipv6("::"))
  expect_true(is_ipv6("2001:db8::1"))
  expect_true(is_ipv6("2001:db8::8a2e:370:7334"))
  expect_true(is_ipv6("fe80::1"))
  expect_true(is_ipv6("1:2:3:4:5:6:7::"))
  expect_true(is_ipv6("::1:2:3:4:5:6:7"))
  expect_true(is_ipv6("::ffff:192.168.1.1"))
  expect_true(is_ipv6("2001:db8::192.168.1.1"))
  expect_true(is_ipv6("::1.2.3.4"))
  expect_true(is_ipv6("1:2:3:4:5:6:1.2.3.4"))

  expect_false(is_ipv6("2001:db8::8a2e::7334"))
  expect_false(is_ipv6("2001:0db8:85a3:0000:0000:8a2e:0370"))
  expect_false(is_ipv6("2001:0db8:85a3:0000:0000:8a2e:0370:7334:1234"))
  expect_false(is_ipv6("2001:0db8:85a3:0000:0000:8a2e:0370:gggg"))
  expect_false(is_ipv6("::12345"))
  expect_false(is_ipv6("::ffff:999.168.1.1"))
  expect_false(is_ipv6("1:2:3:4:5:6:1.2.3"))
  expect_false(is_ipv6("1.2.3.4::"))
  expect_false(is_ipv6("192.168.1.1"))
  expect_false(is_ipv6("1:2:3:4:5:6:7:8:"))
  expect_false(is_ipv6(":1:2:3:4:5:6:7:8"))
  expect_false(is_ipv6("a:b:c:d:e:f:0:1::"))
  expect_false(is_ipv6(":::"))
  expect_false(is_ipv6("1:::2"))
  expect_false(is_ipv6(":"))
  expect_false(is_ipv6(""))
  expect_false(is_ipv6(1L))
})

test_that("is_hex works", {
  expect_true(is_hex("1a2b3c"))
  expect_true(is_hex("DEADBEEF"))
  expect_true(is_hex("0"))
  expect_true(is_hex("abcdef0123456789"))

  expect_false(is_hex("")) # empty
  expect_false(is_hex("0x1a2b")) # prefix
  expect_false(is_hex("ghijkl")) # non-hex chars
  expect_false(is_hex("1a 2b")) # spaces
  expect_false(is_hex(123L))
})

test_that("is_md5 works", {
  expect_true(is_md5("d41d8cd98f00b204e9800998ecf8427e"))
  expect_true(is_md5("D41D8CD98F00B204E9800998ECF8427E"))

  expect_false(is_md5("d41d8cd98f00b204e9800998ecf8427")) # too short
  expect_false(is_md5("d41d8cd98f00b204e9800998ecf8427ea")) # too long
  expect_false(is_md5("g41d8cd98f00b204e9800998ecf8427e")) # non-hex
})

test_that("is_sha256 works", {
  expect_true(is_sha256("e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"))
  expect_true(is_sha256("E3B0C44298FC1C149AFBF4C8996FB92427AE41E4649B934CA495991B7852B855"))

  expect_false(is_sha256("e3b0c44298fc1c149afbf4c8996fb924")) # too short (md5)
  expect_false(is_sha256("g3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855")) # non-hex
})

test_that("is_semver works", {
  expect_true(is_semver("1.0.0"))
  expect_true(is_semver("0.1.0"))
  expect_true(is_semver("1.0.0-alpha"))
  expect_true(is_semver("1.0.0-alpha.1"))
  expect_true(is_semver("1.0.0+build.123"))
  expect_true(is_semver("1.0.0-alpha.1+build.123"))
  expect_true(is_semver("10.20.30"))

  expect_false(is_semver("1.0")) # missing patch
  expect_false(is_semver("1")) # just major
  expect_false(is_semver("01.0.0")) # leading zero
  expect_false(is_semver("1.0.0-")) # trailing hyphen
  expect_false(is_semver("1.0.0+")) # trailing plus
  expect_false(is_semver("v1.0.0")) # v prefix
})

test_that("is_hostname works", {
  expect_true(is_hostname("example.com"))
  expect_true(is_hostname("sub.domain.example.com"))
  expect_true(is_hostname("localhost"))
  expect_true(is_hostname("my-host"))
  expect_true(is_hostname("a"))

  expect_false(is_hostname("-example.com")) # starts with hyphen
  expect_false(is_hostname("example-.com")) # ends with hyphen
  expect_false(is_hostname("exam ple.com")) # space
  expect_false(is_hostname(".example.com")) # leading dot
  expect_false(is_hostname("")) # empty
  expect_false(is_hostname(paste(rep("a", 254), collapse = ""))) # too long
})

test_that("is_mac works", {
  expect_true(is_mac("00:1B:44:11:3A:B7"))
  expect_true(is_mac("aa:bb:cc:dd:ee:ff"))
  expect_true(is_mac("AA-BB-CC-DD-EE-FF"))

  expect_false(is_mac("00:1B:44:11:3A")) # too short
  expect_false(is_mac("00:1B:44:11:3A:B7:00")) # too long
  expect_false(is_mac("00:1B:44:11:3A:GG")) # invalid hex
  expect_false(is_mac("00:1B:44-11:3A:B7")) # mixed separators
  expect_false(is_mac("001B44113AB7")) # no separators
})

test_that("is_ulid works", {
  expect_true(is_ulid("01ARZ3NDEKTSV4RRFFQ69G5FAV"))
  expect_true(is_ulid("01H5V6E3MXHP0GGQB7K8CPWM1A"))

  expect_false(is_ulid("01ARZ3NDEKTSV4RRFFQ69G5FA")) # too short
  expect_false(is_ulid("01ARZ3NDEKTSV4RRFFQ69G5FAVX")) # too long
  expect_false(is_ulid("01ARZ3NDEKTSV4RRFFQ69G5FAi")) # lowercase
  expect_false(is_ulid("01ARZ3NDEKTSV4RRFFQ69G5FAO")) # invalid char O
})

test_that("is_nanoid works", {
  expect_true(is_nanoid("V1StGXR8_Z5jdHi6B-myT"))
  expect_true(is_nanoid("012345678901234567890"))

  expect_false(is_nanoid("V1StGXR8_Z5jdHi6B-my")) # too short
  expect_false(is_nanoid("V1StGXR8_Z5jdHi6B-myTx")) # too long
  expect_false(is_nanoid("V1StGXR8_Z5jdHi6B my")) # space
})

test_that("is_cuid2 works", {
  expect_true(is_cuid2("ckopqwooh000001la8mbi2im9"))
  expect_true(is_cuid2("abcdefghijklmnopqrstuvwx"))

  expect_false(is_cuid2("Akopqwooh000001la8mbi2im9")) # starts uppercase
  expect_false(is_cuid2("1kopqwooh000001la8mbi2im9")) # starts with digit
  expect_false(is_cuid2("abc")) # too short
})

#' Check if an argument is a valid IBAN string
#'
#' Validates IBAN format including MOD-97-10 check digit verification (ISO/IEC 7064).
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid IBAN string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/International_Bank_Account_Number>
#' @examples
#' is_iban("GB29NWBK60161331926819")
#' @export
is_iban <- function(x) {
  if (!is_string(x) || !grepl("^[A-Z]{2}[0-9]{2}[A-Z0-9]{11,30}$", x)) {
    return(FALSE)
  }
  codes <- utf8ToInt(x)
  n <- length(codes)
  codes <- c(codes[5:n], codes[1:4])
  mod97(codes) == 1L
}

#' Check if an argument is a BIC/SWIFT code string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid BIC/SWIFT code string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/ISO_9362>
#' <https://knowledge.xmldation.com/support/iso20022/general_rules/bic>
#' @examples
#' is_bic("DEUTDEFF")
#' @export
is_bic <- function(x) {
  is_string(x) && grepl("^[A-Z]{4}[A-Z]{2}[A-Z2-9][A-NP-Z0-9]([A-Z0-9]{3})?$", x)
}

#' Check if an argument is a valid CUSIP string
#'
#' Validates CUSIP format including Luhn-variant check digit verification.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid CUSIP string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/CUSIP>
#' @examples
#' is_cusip("037833100")
#' @export
is_cusip <- function(x) {
  if (!is_string(x) || !grepl("^[A-Z0-9]{8}[0-9]$", x)) {
    return(FALSE)
  }
  codes <- utf8ToInt(x)
  vals <- char_to_val(codes[1:8])
  pos <- seq.int(2L, 8L, by = 2L)
  vals[pos] <- vals[pos] * 2L
  total <- sum(vals %/% 10L + vals %% 10L)
  check <- (10L - total %% 10L) %% 10L
  codes[9L] - 48L == check
}

#' Check if an argument is a valid FIGI string
#'
#' Validates FIGI (Financial Instrument Global Identifier) format including Luhn check digit
#' verification.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid FIGI string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/Financial_Instrument_Global_Identifier>
#' <https://www.openfigi.com/about/overview>
#' @examples
#' is_figi("BBG000BLNNH6")
#' @export
is_figi <- function(x) {
  if (!is_string(x) || !grepl("^[B-DF-HJ-NP-TV-Z]{2}G[B-DF-HJ-NP-TV-Z0-9]{8}[0-9]$", x)) {
    return(FALSE)
  }
  codes <- utf8ToInt(x)
  vals <- char_to_val(codes[1:11])
  pos <- seq.int(2L, 10L, by = 2L)
  vals[pos] <- vals[pos] * 2L
  total <- sum(vals %/% 10L + vals %% 10L)
  check <- (10L - total %% 10L) %% 10L
  codes[12L] - 48L == check
}

#' Check if an argument is a DOI string
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid DOI string, `FALSE` otherwise.
#' @references
#' <https://www.doi.org/the-identifier/what-is-a-doi/>
#' @examples
#' is_doi("10.1038/nphys1170")
#' @export
is_doi <- function(x) {
  regex <- "^10\\.\\d{4,9}/[-._;()/:A-Z0-9]+$"
  is_string(x) && grepl(regex, x, ignore.case = TRUE, perl = TRUE)
}

#' Check if an argument is a valid ISIN string
#'
#' Validates ISIN (International Securities Identification Number) format including Luhn check
#' digit verification.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid ISIN string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/International_Securities_Identification_Number>
#' @examples
#' is_isin("US0378331005")
#' @export
is_isin <- function(x) {
  if (!is_string(x) || !grepl("^[A-Z]{2}[A-Z0-9]{9}[0-9]$", x)) {
    return(FALSE)
  }
  codes <- utf8ToInt(x)
  vals <- char_to_val(codes)
  digits <- integer(24L)
  k <- 0L
  for (v in vals) {
    if (v >= 10L) {
      k <- k + 1L
      digits[k] <- v %/% 10L
      k <- k + 1L
      digits[k] <- v %% 10L
    } else {
      k <- k + 1L
      digits[k] <- v
    }
  }
  digits <- digits[seq_len(k)]
  pos <- seq.int(k - 1L, 1L, by = -2L)
  digits[pos] <- digits[pos] * 2L
  digits[pos] <- ifelse(digits[pos] > 9L, digits[pos] - 9L, digits[pos])
  sum(digits) %% 10L == 0L
}

#' Check if an argument is an ISBN string
#'
#' Validates ISBN-10 and ISBN-13 formats, including check digit verification.
#' Hyphens and spaces are allowed as separators.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid ISBN string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/International_Standard_Book_Number>
#' @examples
#' is_isbn("978-0-306-40615-7")
#' @export
is_isbn <- function(x) {
  if (!is_string(x)) {
    return(FALSE)
  }
  digits <- gsub("[- ]", "", x)
  if (grepl("^\\d{9}[0-9X]$", digits)) {
    codes <- utf8ToInt(digits)
    d <- parse_check_digits(codes)
    sum(d * 10:1) %% 11L == 0L
  } else if (grepl("^\\d{13}$", digits)) {
    d <- utf8ToInt(digits) - 48L
    sum(d * rep_len(c(1L, 3L), 13L)) %% 10L == 0L
  } else {
    FALSE
  }
}

#' Check if an argument is an ISSN string
#'
#' Validates ISSN format including check digit verification.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid ISSN string, `FALSE` otherwise.
#' @references
#' <https://www.loc.gov/issn/check.html>
#' @examples
#' is_issn("0378-5955")
#' @export
is_issn <- function(x) {
  if (!is_string(x) || !grepl("^\\d{4}-\\d{3}[0-9X]$", x)) {
    return(FALSE)
  }
  codes <- utf8ToInt(x)
  d <- parse_check_digits(codes[c(1:4, 6:9)])
  sum(d * 8:1) %% 11L == 0L
}

#' Check if an argument is a valid LEI string
#'
#' Validates LEI (Legal Entity Identifier) format including MOD-97-10 check digit verification
#' (ISO/IEC 7064).
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid LEI string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/Legal_Entity_Identifier>
#' <https://www.govinfo.gov/content/pkg/CFR-2016-title12-vol8/xml/CFR-2016-title12-vol8-part1003-appC.xml>
#' @examples
#' is_lei("7H6GLXDRUGQFU57RNE97")
#' @export
is_lei <- function(x) {
  if (!is_string(x) || !grepl("^[A-Z0-9]{18}[0-9]{2}$", x)) {
    return(FALSE)
  }
  mod97(utf8ToInt(x)) == 1L
}

#' Check if an argument is an ORCID string
#'
#' Validates ORCID format including ISO/IEC 7064:2003, MOD 11-2 check digit
#' verification.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid ORCID string, `FALSE` otherwise.
#' @references
#' <https://support.orcid.org/hc/en-us/articles/360006897674-Structure-of-the-ORCID-Identifier>
#' @examples
#' is_orcid("0000-0002-1825-0097")
#' @export
is_orcid <- function(x) {
  if (!is_string(x) || !grepl("^\\d{4}-\\d{4}-\\d{4}-\\d{3}[0-9X]$", x)) {
    return(FALSE)
  }
  codes <- utf8ToInt(x)
  d <- codes[c(1:4, 6:9, 11:14, 16:19)] - 48L
  total <- 0L
  for (i in seq_len(15L)) {
    total <- (total + d[i]) * 2L
  }
  check <- (12L - total %% 11L) %% 11L
  if (check == 10L) d[16L] + 48L == 88L else d[16L] == check
}

#' Check if an argument is a valid SEDOL string
#'
#' Validates SEDOL format including weighted check digit verification.
#'
#' @param x (`any`)\cr
#'   Object to check.
#' @return `TRUE` if `x` is a valid SEDOL string, `FALSE` otherwise.
#' @references
#' <https://en.wikipedia.org/wiki/SEDOL>
#' @examples
#' is_sedol("0263494")
#' @export
is_sedol <- function(x) {
  if (!is_string(x) || !grepl("^[B-DF-HJ-NP-TV-Z0-9]{6}[0-9]$", x)) {
    return(FALSE)
  }
  codes <- utf8ToInt(x)
  vals <- char_to_val(codes[1:6])
  wt <- c(1L, 3L, 1L, 7L, 3L, 9L)
  total <- sum(vals * wt)
  check <- (10L - total %% 10L) %% 10L
  codes[7L] - 48L == check
}

char_to_val <- function(x) {
  ifelse(x >= 65L, x - 55L, x - 48L)
}

parse_check_digits <- function(x) {
  ifelse(x == 88L, 10L, x - 48L)
}

mod97 <- function(codes) {
  vals <- ifelse(codes >= 65L, codes - 55L, codes - 48L)
  rem <- 0L
  for (v in vals) {
    if (v >= 10L) {
      rem <- (rem * 100L + v) %% 97L
    } else {
      rem <- (rem * 10L + v) %% 97L
    }
  }
  rem
}

test_that("is_iban works", {
  expect_true(is_iban("GB29NWBK60161331926819"))
  expect_true(is_iban("DE89370400440532013000"))
  expect_true(is_iban("FR7630006000011234567890189"))

  expect_false(is_iban("GB29NWBK60161331926818")) # bad check digits
  expect_false(is_iban("DE00370400440532013000")) # check digits 00
  expect_false(is_iban("1234567890")) # no country code
  expect_false(is_iban("GB29")) # too short
  expect_false(is_iban("gb29NWBK60161331926819")) # lowercase country
  expect_false(is_iban(123L))
})

test_that("is_bic works", {
  expect_true(is_bic("DEUTDEFF"))
  expect_true(is_bic("DEUTDEFF500"))
  expect_true(is_bic("BNPAFRPP"))
  expect_true(is_bic("CHASUS33XXX"))

  expect_false(is_bic("DEUT")) # too short
  expect_false(is_bic("DEUTDEFF5000")) # too long
  expect_false(is_bic("DEU1DEFF")) # digit in institution code
  expect_false(is_bic("DEUTD1FF")) # digit in country code
  expect_false(is_bic("DEUTDE0F")) # 0 not allowed in location position 7
  expect_false(is_bic("DEUTDE1F")) # 1 not allowed in location position 7
  expect_false(is_bic("DEUTDEFO")) # O not allowed in location position 8
  expect_false(is_bic("deutdeff")) # lowercase
  expect_false(is_bic(123L))
})

test_that("is_cusip works", {
  expect_true(is_cusip("037833100")) # Apple
  expect_true(is_cusip("17275R102")) # Cisco
  expect_true(is_cusip("594918104")) # Microsoft

  expect_false(is_cusip("037833101")) # bad check digit
  expect_false(is_cusip("03783310")) # too short
  expect_false(is_cusip("0378331000")) # too long
  expect_false(is_cusip("03783310a")) # lowercase
  expect_false(is_cusip("03783310A")) # letter in check digit position
  expect_false(is_cusip(123L))
})

test_that("is_figi works", {
  expect_true(is_figi("BBG000BLNNH6")) # IBM
  expect_true(is_figi("BBG000B9XRY4")) # Apple

  expect_false(is_figi("BBG000BLNNH7")) # bad check digit
  expect_false(is_figi("BBG000BLNNH")) # too short
  expect_false(is_figi("BBG000BLNNH60")) # too long
  expect_false(is_figi("ABG000BLNNH6")) # vowel A in prefix
  expect_false(is_figi("BBB000BLNNH6")) # 3rd char must be G
  expect_false(is_figi("BBG000BLANH6")) # vowel A not allowed
  expect_false(is_figi(123L))
})

test_that("is_doi works", {
  expect_true(is_doi("10.1000/xyz123"))
  expect_true(is_doi("10.1038/nphys1170"))
  expect_true(is_doi("10.1002/0470841559.ch1"))
  expect_true(is_doi("10.1093/ajae/aaq063"))

  expect_false(is_doi("11.1000/xyz123")) # wrong prefix
  expect_false(is_doi("10.12/xyz")) # registrant too short
  expect_false(is_doi("10.1000/")) # empty suffix
  expect_false(is_doi("10.1000")) # no slash
})

test_that("is_isin works", {
  expect_true(is_isin("US0378331005")) # Apple
  expect_true(is_isin("GB0002634946")) # BAE Systems
  expect_true(is_isin("DE0005810055")) # Deutsche Boerse

  expect_false(is_isin("US0378331006")) # bad check digit
  expect_false(is_isin("US037833100")) # too short
  expect_false(is_isin("US03783310050")) # too long
  expect_false(is_isin("us0378331005")) # lowercase
  expect_false(is_isin(123L))
})

test_that("is_isbn works", {
  # ISBN-10
  expect_true(is_isbn("0306406152"))
  expect_true(is_isbn("0-306-40615-2"))
  expect_true(is_isbn("080442957X"))
  # ISBN-13
  expect_true(is_isbn("9780306406157"))
  expect_true(is_isbn("978-0-306-40615-7"))
  expect_true(is_isbn("978 0 306 40615 7"))

  expect_false(is_isbn("0306406153")) # bad check digit
  expect_false(is_isbn("9780306406158")) # bad check digit
  expect_false(is_isbn("030640615")) # too short
  expect_false(is_isbn("12345")) # wrong length
  expect_false(is_isbn(123L))
})

test_that("is_issn works", {
  expect_true(is_issn("0378-5955"))
  expect_true(is_issn("0317-8471"))
  expect_true(is_issn("1050-124X"))

  expect_false(is_issn("0378-5956")) # bad check digit
  expect_false(is_issn("03785955")) # no hyphen
  expect_false(is_issn("0378-595")) # too short
  expect_false(is_issn(123L))
})

test_that("is_lei works", {
  expect_true(is_lei("7H6GLXDRUGQFU57RNE97"))
  expect_true(is_lei("529900T8BM49AURSDO55"))
  expect_true(is_lei("213800WSGIIZCXF1P572"))

  expect_false(is_lei("7H6GLXDRUGQFU57RNE98")) # bad check digits
  expect_false(is_lei("7H6GLXDRUGQFU57RNE9")) # too short
  expect_false(is_lei("7H6GLXDRUGQFU57RNE970")) # too long
  expect_false(is_lei("7h6glxdrugqfu57rne97")) # lowercase
  expect_false(is_lei(123L))
})

test_that("is_sedol works", {
  expect_true(is_sedol("0263494")) # BAE Systems
  expect_true(is_sedol("B0YBKJ7")) # Vodafone
  expect_true(is_sedol("0540528")) # GlaxoSmithKline

  expect_false(is_sedol("0263495")) # bad check digit
  expect_false(is_sedol("026349")) # too short
  expect_false(is_sedol("02634940")) # too long
  expect_false(is_sedol("A263494")) # vowel A not allowed
  expect_false(is_sedol(123L))
})

test_that("is_orcid works", {
  # sample ORCIDs from the ORCID website
  expect_true(is_orcid("0000-0002-1825-0097"))
  expect_true(is_orcid("0000-0001-5109-3700"))
  expect_true(is_orcid("0000-0002-1694-233X"))
  expect_true(is_orcid("0000-0001-5000-0007"))

  expect_false(is_orcid("0000-0002-1825-0098")) # bad check digit
  expect_false(is_orcid("0000-0003-0918-376")) # too short
  expect_false(is_orcid("0000-0003-0918-37666")) # too long
  expect_false(is_orcid("000000030918376")) # no hyphens
  expect_false(is_orcid("0000-0003-0918-376A")) # invalid check char
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id.R
\name{is_orcid}
\alias{is_orcid}
\title{Check if an argument is an ORCID string}
\usage{
is_orcid(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid ORCID string, \code{FALSE} otherwise.
}
\description{
Validates ORCID format including ISO/IEC 7064:2003, MOD 11-2 check digit
verification.
}
\examples{
is_orcid("0000-0002-1825-0097")
}
\references{
\url{https://support.orcid.org/hc/en-us/articles/360006897674-Structure-of-the-ORCID-Identifier}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_ipv6}
\alias{is_ipv6}
\title{Check if an argument is an IPv6 address string}
\usage{
is_ipv6(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid IPv6 address string, \code{FALSE} otherwise.
}
\description{
Validates full, compressed (\code{::}) and IPv4-embedded (\verb{::ffff:192.168.1.1}) forms.
Zone IDs (e.g. \verb{\%eth0}) are not supported.
}
\examples{
is_ipv6("2001:0db8:85a3:0000:0000:8a2e:0370:7334")
is_ipv6("::1")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_mime}
\alias{is_mime}
\title{Check if an argument is a MIME type string}
\usage{
is_mime(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid MIME type string, \code{FALSE} otherwise.
}
\description{
Validates the \code{type/subtype} format where the top-level type must be one of the IANA-registered
types: \code{application}, \code{audio}, \code{font}, \code{image}, \code{message}, \code{model}, \code{multipart}, \code{text}, or
\code{video}.
}
\examples{
is_mime("application/json")
is_mime("text/plain")
}
\references{
\url{https://www.iana.org/assignments/media-types/media-types.xhtml}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id.R
\name{is_issn}
\alias{is_issn}
\title{Check if an argument is an ISSN string}
\usage{
is_issn(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid ISSN string, \code{FALSE} otherwise.
}
\description{
Validates ISSN format including check digit verification.
}
\examples{
is_issn("0378-5955")
}
\references{
\url{https://www.loc.gov/issn/check.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_sha256}
\alias{is_sha256}
\title{Check if an argument is a SHA-256 hash string}
\usage{
is_sha256(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid SHA-256 hash string, \code{FALSE} otherwise.
}
\description{
Check if an argument is a SHA-256 hash string
}
\examples{
is_sha256("e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_semver}
\alias{is_semver}
\title{Check if an argument is a semantic versioning string}
\usage{
is_semver(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid semver string, \code{FALSE} otherwise.
}
\description{
Validates semantic versioning 2.0.0 format, including optional pre-release and build metadata.
Uses the official recommended regex from \url{https://semver.org/}.
}
\examples{
is_semver("1.0.0")
}
\references{
\url{https://semver.org/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_color_hex}
\alias{is_color_hex}
\title{Check if an argument is a hex color string}
\usage{
is_color_hex(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid hex color string, \code{FALSE} otherwise.
}
\description{
Validates hex color codes in \verb{#RGB}, \verb{#RGBA}, \verb{#RRGGBB}, or \verb{#RRGGBBAA} format.
}
\examples{
is_color_hex("#FF5733")
is_color_hex("#fff")
}
\references{
\url{https://www.w3.org/TR/css-color-4/#hex-notation}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id.R
\name{is_doi}
\alias{is_doi}
\title{Check if an argument is a DOI string}
\usage{
is_doi(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid DOI string, \code{FALSE} otherwise.
}
\description{
Check if an argument is a DOI string
}
\examples{
is_doi("10.1038/nphys1170")
}
\references{
\url{https://www.doi.org/the-identifier/what-is-a-doi/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assertions.R
\name{assert_empty_dots}
\alias{assert_empty_dots}
\title{Assert that dots are empty}
\usage{
assert_empty_dots(...)
}
\arguments{
\item{...}{(\code{any})\cr
Ellipsis arguments to check.}
}
\value{
\code{NULL} invisibly if empty, otherwise throws an error.
}
\description{
Assert that dots are empty
}
\examples{
assert_empty_dots()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id.R
\name{is_lei}
\alias{is_lei}
\title{Check if an argument is a valid LEI string}
\usage{
is_lei(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid LEI string, \code{FALSE} otherwise.
}
\description{
Validates LEI (Legal Entity Identifier) format including MOD-97-10 check digit verification
(ISO/IEC 7064).
}
\examples{
is_lei("7H6GLXDRUGQFU57RNE97")
}
\references{
\url{https://en.wikipedia.org/wiki/Legal_Entity_Identifier}
\url{https://www.govinfo.gov/content/pkg/CFR-2016-title12-vol8/xml/CFR-2016-title12-vol8-part1003-appC.xml}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id.R
\name{is_figi}
\alias{is_figi}
\title{Check if an argument is a valid FIGI string}
\usage{
is_figi(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid FIGI string, \code{FALSE} otherwise.
}
\description{
Validates FIGI (Financial Instrument Global Identifier) format including Luhn check digit
verification.
}
\examples{
is_figi("BBG000BLNNH6")
}
\references{
\url{https://en.wikipedia.org/wiki/Financial_Instrument_Global_Identifier}
\url{https://www.openfigi.com/about/overview}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id.R
\name{is_cusip}
\alias{is_cusip}
\title{Check if an argument is a valid CUSIP string}
\usage{
is_cusip(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid CUSIP string, \code{FALSE} otherwise.
}
\description{
Validates CUSIP format including Luhn-variant check digit verification.
}
\examples{
is_cusip("037833100")
}
\references{
\url{https://en.wikipedia.org/wiki/CUSIP}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_uuid}
\alias{is_uuid}
\title{Check if an argument is an UUID string}
\usage{
is_uuid(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid UUID string, \code{FALSE} otherwise.
}
\description{
Check if an argument is an UUID string
}
\examples{
is_uuid("550e8400-e29b-41d4-a716-446655440000")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_ulid}
\alias{is_ulid}
\title{Check if an argument is a ULID string}
\usage{
is_ulid(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid ULID string, \code{FALSE} otherwise.
}
\description{
Check if an argument is a ULID string
}
\examples{
is_ulid("01ARZ3NDEKTSV4RRFFQ69G5FAV")
}
\references{
\url{https://github.com/ulid/spec}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id.R
\name{is_bic}
\alias{is_bic}
\title{Check if an argument is a BIC/SWIFT code string}
\usage{
is_bic(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid BIC/SWIFT code string, \code{FALSE} otherwise.
}
\description{
Check if an argument is a BIC/SWIFT code string
}
\examples{
is_bic("DEUTDEFF")
}
\references{
\url{https://en.wikipedia.org/wiki/ISO_9362}
\url{https://knowledge.xmldation.com/support/iso20022/general_rules/bic}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_iso_datetime}
\alias{is_iso_datetime}
\title{Check if an argument is an ISO 8601 datetime string}
\usage{
is_iso_datetime(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid ISO 8601 datetime string, \code{FALSE} otherwise.
}
\description{
Validates the \code{YYYY-MM-DDTHH:MM:SS[.sss]} format with calendar correctness, including leap year
handling. The timezone designator is optional and can be \code{Z} or an offset like \code{+05:30}.
}
\examples{
is_iso_datetime("2024-01-15T12:00:00Z")
is_iso_datetime("2024-01-15T12:00:00+05:30")
is_iso_datetime("2024-01-15T12:00:00")
}
\references{
\url{https://en.wikipedia.org/wiki/ISO_8601}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id.R
\name{is_isbn}
\alias{is_isbn}
\title{Check if an argument is an ISBN string}
\usage{
is_isbn(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid ISBN string, \code{FALSE} otherwise.
}
\description{
Validates ISBN-10 and ISBN-13 formats, including check digit verification.
Hyphens and spaces are allowed as separators.
}
\examples{
is_isbn("978-0-306-40615-7")
}
\references{
\url{https://en.wikipedia.org/wiki/International_Standard_Book_Number}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_base64url}
\alias{is_base64url}
\title{Check if an argument is base64url string}
\usage{
is_base64url(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid base64url string, \code{FALSE} otherwise.
}
\description{
Check if an argument is base64url string
}
\examples{
is_base64url("SGVsbG8gV29ybGQ")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id.R
\name{is_isin}
\alias{is_isin}
\title{Check if an argument is a valid ISIN string}
\usage{
is_isin(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid ISIN string, \code{FALSE} otherwise.
}
\description{
Validates ISIN (International Securities Identification Number) format including Luhn check
digit verification.
}
\examples{
is_isin("US0378331005")
}
\references{
\url{https://en.wikipedia.org/wiki/International_Securities_Identification_Number}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string.R
\name{is_iso_date}
\alias{is_iso_date}
\title{Check if an argument is an ISO 8601 date string}
\usage{
is_iso_date(x)
}
\arguments{
\item{x}{(\code{any})\cr
Object to check.}
}
\value{
\code{TRUE} if \code{x} is a valid ISO 8601 date string, \code{FALSE} otherwise.
}
\description{
Validates the \code{YYYY-MM-DD} format with calendar correctness, including leap year handling.
}
\examples{
is_iso_date("2024-01-15")
is_iso_date("2024-02-30")
}
\references{
\url{https://en.wikipedia.org/wiki/ISO_8601}
}
